-- DailyQuestHelper Version 0.1
-- author alexwild
DailyQuestHelper = LibStub("AceAddon-3.0"):NewAddon("DailyQuestHelper", "AceConsole-3.0", "AceEvent-3.0")

local questEvent = "";
local options;

function DailyQuestHelper:OnInitialize()    
    local defaults = DailyQuestHelper:CreateDefaults();
    self.db = LibStub("AceDB-3.0"):New("DailyQuestHelperDB", defaults, "Default");
    options = DailyQuestHelper:CreateQuestOptions(options);
    LibStub("AceConfig-3.0"):RegisterOptionsTable("DailyQuestHelper", options);	
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DailyQuestHelper", "DailyQuestHelper");
    LibStub("AceConfigDialog-3.0"):SetDefaultSize("DailyQuestHelper", 700, 400)
    self:RegisterChatCommand("DailyQuestHelper", "ChatCommand");
    self:RegisterChatCommand("dqh", "ChatCommand");
    DailyQuestHelper.DEBUG = false;
end

function DailyQuestHelper:OnEnable()
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("QUEST_PROGRESS");
    self:RegisterEvent("QUEST_COMPLETE");
    self:RegisterEvent("QUEST_DETAIL");
end

function DailyQuestHelper:ChatCommand(input)
    if not input or input:trim() == "" then
        --InterfaceOptionsFrame_OpenToFrame(self.optionsFrame);
        LibStub("AceConfigDialog-3.0"):Open("DailyQuestHelper");
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(DailyQuestHelper, "dqh", "DailyQuestHelper", input);
    end
end

function DailyQuestHelper:OnDisable()
end

function DailyQuestHelper:isDebugMode(info)
    return self.DEBUG;
end

function DailyQuestHelper:toggleDebugMode(info, value)
    self.DEBUG = not self.DEBUG;
end

----------------------------------------
-- event GOSSIP_SHOW, auto accept Quests
----------------------------------------
function DailyQuestHelper:GOSSIP_SHOW()
    questEvent = "GOSSIP_SHOW";
    if self.db.char.enable then
        DailyQuestHelper:AcceptQuest();
    end
end

---------------------------------------------
-- event Quest_COMPLETE, auto complete Quests
---------------------------------------------
function DailyQuestHelper:QUEST_COMPLETE()
    questEvent = "QUEST_COMPLETE";
    if self.db.char.enable then
		DailyQuestHelper:QuestComplete();
	end
end
 
-----------------------------------------
-- event QUEST_DETAIL, auto accept Quests
-----------------------------------------
function DailyQuestHelper:QUEST_DETAIL()
    questEvent = "QUEST_DETAIL";
    if self.db.char.enable then
        DailyQuestHelper:QuestDetail();
    end
end

-------------------------------------------
-- event QUEST_PROGRESS, auto accept Quests
-------------------------------------------
function DailyQuestHelper:QUEST_PROGRESS()
    questEvent = "QUEST_PROGRESS";
    if self.db.char.enable then
        DailyQuestHelper:QuestProgress();
    end
end

---------------------
-- auto accept Quests
---------------------
function DailyQuestHelper:AcceptQuest()
	if (self.DEBUG) then
	  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): questEvent: "..questEvent);
	end
	-- When the Gossip Window is shown
	if questEvent == "GOSSIP_SHOW" then
	
		local activeQuests = {GetGossipActiveQuests()};
			
		local numOfAcQuests = GetNumGossipActiveQuests();
			
		if (numOfAcQuests > 0) then
			if (self.DEBUG) then
				DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): Number of Active Quest "..numOfAcQuests);
			end
			for numberOfQuestToAnalyse = 1, numOfAcQuests, 1 do
				local indexOfQuest = (numberOfQuestToAnalyse-1)*4;
				if (self.db.char.quests[activeQuests[indexOfQuest + 1]]) then
					if (activeQuests[indexOfQuest +4]) then
						SelectGossipActiveQuest(numberOfQuestToAnalyse);
					end
				end				
			end
		end
		-- get the table of the available quest gossips of the npc
		local npcQuests = {GetGossipAvailableQuests()};
		
		-- because the questgossips are in a table as multiple result, the number of available quests are needed
		local numberOfQuests = GetNumGossipAvailableQuests();
		if (self.DEBUG) then
			DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): Number of Quest "..numberOfQuests);
		end
		
		-- the number of quests are used to calculate the index of the start for a new quest in the table.
		for numberOfQuestToAnalyse = 1, numberOfQuests, 1 do
			-- title, level, isLowLevel, isDaily, isRepeatable are the resutlts for a quest.
			local indexOfQuest = (numberOfQuestToAnalyse-1)*5;
			if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): Index of Quest: "..indexOfQuest);
			end
			if (npcQuests[indexOfQuest + 4]) then
			-- the quest is a daily quest.
			local questTitle = npcQuests[indexOfQuest + 1];
				if (self.DEBUG) then
					DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): Quest is Daily "..questTitle);
				end
				if (self.db.char.quests[questTitle] == nil) then
					self.db.char.quests[questTitle] = {};
					self.db.char.quests[questTitle].active = true;
					self.db.char.quests[questTitle].location = GetRealZoneText();
					DailyQuestHelper:UpdateQuestOptions();
				end
				if (self.db.char.quests[questTitle].active) then
					SelectGossipAvailableQuest(numberOfQuestToAnalyse);
					CompleteQuest();
				end
			end
		end
	end
end

function DailyQuestHelper:QuestDetail()
	-- When the Quest Details are shown
	if questEvent == "QUEST_DETAIL" then		
		-- check if the shown quest is a daily quest.
		if (QuestIsDaily()) then
			-- the quest is a daily quest, so accect that quest.
			local questTitle = GetTitleText();

			if (self.db.char.quests[questTitle] == nil) then
				self.db.char.quests[questTitle] = {};
				self.db.char.quests[questTitle].active = true;
				self.db.char.quests[questTitle].location = GetRealZoneText();
				DailyQuestHelper:UpdateQuestOptions();
			end
			if (self.db.char.quests[questTitle].active) then
				AcceptQuest();
			end
		end
	end
end

function DailyQuestHelper:QuestComplete()
	if (self.DEBUG) then
	  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): questEvent: "..questEvent);
	end
	if (QuestIsDaily()) then
		-- autocomplete quest
		if (GetNumQuestChoices() > 1) then
		-- if there are more items to choose of, then don't autoturnin
		else
			GetQuestReward(0)
		end
	end
end

function DailyQuestHelper:QuestProgress()
	if (self.DEBUG) then
	  DEFAULT_CHAT_FRAME:AddMessage("DEBUG(AcceptQuest()): questEvent: "..questEvent);
	end
	if (QuestIsDaily()) then
		-- the quest is a daily quest, so accect that quest.
		if (IsQuestCompletable()) then
			if (self.DEBUG) then
				DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QuestProgress()): Quest is completable.");				
			end
			CompleteQuest();
		else
			if (self.DEBUG) then
				DEFAULT_CHAT_FRAME:AddMessage("DEBUG(QuestProgress()): Quest not completable.");
			end
		end
	end
end

function DailyQuestHelper:UpdateQuestOptions()
	
end

----------------------------------------------------------
-- create the options to enable oder disable single quests
----------------------------------------------------------
function DailyQuestHelper:CreateQuestOptions()
options = {
    name = "DailyQuestHelper",
    handler = DailyQuestHelper,
    type = 'group',
    args = {
        general = {
            name = "General",
            type = "group",
            args = {
                activate = {
                    name = "enable addon",
                    type = "toggle",
                    desc = "enable addon",
                    get = function()
							return self.db.char.enable;
                            end,
                    set = function(i, v)
                            self.db.char.enable = v;
                            end,
                },
--			    reset = {
--                    name = "remove all quests",
--                    type = "execute",
--                    desc = "remove all quests",
--                    func = function()
--                           self.db.char.quests= {};
--                            end,
                },
--                debug = {
--                    name = "debugmode",
--                    type = "toggle",
--                    desc = "enable/disable debugmode",
--                    get = "isDebugMode",
--                    set = "toggleDebugMode",
--                },
            },
        },
        quests = {
            name = "Quests",
            type = "group",
        },
    },
}

    local option = {};
	for questKey, questValues in pairs(self.db.char.quests) do
		
		local button = {};
		if (option[questKey]) then
			button = option[questKey];			
		else
			button.name = questKey;

			button.type = "toggle";
			local description = "";

			button.desc = description;
			button.get = function()							
							return self.db.char.quests[questKey].active;							
						end
			button.set = function(i, v)
						self.db.char.quests[questKey].active = v;
						end		
			end
		option[questKey] = button;
	end
    options.args.quests.args = option;
	return options;
end

-----------------------------------------------------
-- get the key for a given value (only the first key)
-----------------------------------------------------
function DailyQuestHelper:GetKey(table, text)
    for key, value in pairs(table) do                
        if value == text then
            return key;
        end
    end
end

-------------------------
-- create default options
-------------------------
function DailyQuestHelper:CreateDefaults()
    local defaults = {
        char = {
            debug = false,
            enable = true,
			quests = {},
        },
    }
    return defaults;
end